// PumpControllerV1
// -*- mode: C++ -*-
//
// Shows how to run AccelStepper in the simplest,
// fixed speed mode with no accelerations
// Requires the Adafruit_Motorshield v2 library
//   https://github.com/adafruit/Adafruit_Motor_Shield_V2_Library
// And AccelStepper with AFMotor support
//   https://github.com/adafruit/AccelStepper

// This tutorial is for Adafruit Motorshield v2 only!
// Will not work with v1 shields

#include <AccelStepper.h>
#include <Wire.h>
#include <Adafruit_MotorShield.h>
#include "utility/Adafruit_PWMServoDriver.h"

// Create the motor shield object with the default I2C address
Adafruit_MotorShield AFMS1 = Adafruit_MotorShield(0x60); //motor shield 1
Adafruit_MotorShield AFMS2 = Adafruit_MotorShield(0x61); //motor shield 2



// Connect a stepper motors with 200 steps per revolution (1.8 degree)
Adafruit_StepperMotor *myStepper1 = AFMS1.getStepper(200, 1);//shield 1, port 1 (M1 & M2)
Adafruit_StepperMotor *myStepper2 = AFMS1.getStepper(200, 2);//shield 1, port 2 (M3 & M4)

Adafruit_StepperMotor *myStepper3 = AFMS2.getStepper(200, 1);//shield 2, port 1
//Adafruit_StepperMotor *myStepper4 = AFMS2.getStepper(200, 2);//shield 2, port 2

// wrappers for the first motor
void forwardstep1() {
  myStepper1->onestep(FORWARD, MICROSTEP);
}
void backwardstep1() {
  myStepper1->onestep(BACKWARD, MICROSTEP);
}

// wrappers for the second motor
void forwardstep2() {
  myStepper2->onestep(FORWARD, MICROSTEP);
}
void backwardstep2() {
  myStepper2->onestep(BACKWARD, MICROSTEP);
}

// wrappers for the third motor
void forwardstep3() {
  myStepper3->onestep(FORWARD, MICROSTEP);
}
void backwardstep3() {
  myStepper3->onestep(BACKWARD, MICROSTEP);
}

// wrappers for the fourth motor
//void forwardstep4() {
//myStepper4->onestep(FORWARD, MICROSTEP);
//}
//void backwardstep4() {
//myStepper4->onestep(BACKWARD, MICROSTEP);
//}

AccelStepper Astepper1(forwardstep1, backwardstep1);
AccelStepper Astepper2(forwardstep2, backwardstep2);
AccelStepper Astepper3(forwardstep3, backwardstep3);
//AccelStepper Astepper4(forwardstep4, backwardstep4);


void setup()
{
  // set up Serial library at 9600 bps
  Serial.begin(9600); //port1
  Serial.println("Enter Command [Start, Stop, Release, StartAll, StopAll, ReleaseAll]");

  AFMS1.begin();  // Start first shield @ default frequency 1.6KHz
  AFMS2.begin();//start second shield

  myStepper1->setSpeed(100);
  myStepper2->setSpeed(100);
  myStepper3->setSpeed(100);
  //myStepper4->setSpeed(100);
}

boolean stringComplete = false;
String readString = "", substring; 
int progMode = 0; //0 is main command, 1 is which motor to stop, 2 is which motor to release
int mtr;
float mtrspeed;
boolean mtrrun1 = false; 
boolean mtrrun2 = false;
boolean mtrrun3 = false;
//boolean mtrrun4 = false;


void loop()
{
  if (mtrrun1) {
    Astepper1.runSpeed();
  }
  if (mtrrun2) {
    Astepper2.runSpeed();
  }
  if (mtrrun3) {
    Astepper3.runSpeed();
  }
  //if (mtrrun4) {
  //Astepper4.runSpeed();
  //}
}

void serialEvent() {
  if (Serial.available()) {
   
    char c = Serial.read(); //gets one byte from serial buffer
    if (c != '\n') {
      readString += c; //Add the character to the readString.
      //Serial.print(c); //Print out each character so you can see what you're typing into the console.
      stringComplete = false;
    }
    else { // This will only happen when enter is pressed
      stringComplete = true;
    }

    if (stringComplete) { //The rest of these switch statements are only needed after enter is pressed. 
  
      switch (progMode) {
        case 0:  //main menu
          Serial.print("Command is "); 
          Serial.println(readString); //Now print the command
          if (readString == "Stop") {
            Serial.println("Which motor to stop? [1 , 2 , 3] ");
            readString = ""; //clears variable for new input
            progMode = 1;
          }
          else if (readString == "Release") {
            Serial.println("Which motor to release? [1 , 2 ,3] ");
            readString = ""; //clears variable for new input
            progMode = 2;
          }
          else if (readString == "StopAll") {
            Serial.println("Stopping all motors.");
            mtrrun1 = false;
            mtrrun2 = false;
            mtrrun3 = false;
            //mtrrun4=false;
            readString = ""; //clears variable for new input
            progMode = 0;
            Serial.println("\nEnter Command [Start, Stop, Release, StartAll, StopAll, ReleaseAll]");
          }
          else if (readString == "StartAll") {  
            Serial.println("Starting all motors.");
            Serial.println("What speed?"); 
            readString = "";  
            progMode = 5; 
          }
          else if (readString == "ReleaseAll") {
            Serial.println("Releasing all motors.");
            myStepper1->release();
            myStepper2->release();
            myStepper3->release();
            //myStepper4->release();
  
            readString = ""; //clears variable for new input
            progMode = 0;
            Serial.println("\nEnter Command [Start, Stop, Release, StartAll, StopAll, ReleaseAll]");
          }
          else if (readString == "Start") {
            Serial.println("Which motor to start?"); 
            readString = ""; //clears variable for new input
            progMode = 3;
          }
          else {
            Serial.println("Not a command. ");
            readString = ""; //clears variable for new input
            Serial.println("\nEnter Command [Start, Stop, Release, StartAll, StopAll, ReleaseAll]");
          }
          break;
        case 1:
          switch (readString.toInt()) {
            case 1:
              mtrrun1 = false; //stop motor 1
              Serial.print("Stopping motor ");
              Serial.println(readString.toInt());
              break;
            case 2:
              mtrrun2 = false; // stop motor 2
              Serial.print("Stopping motor ");
              Serial.println(readString.toInt());
              break;
            case 3:
              mtrrun3 = false; // stop motor 3
              Serial.print("Stopping motor ");
              Serial.println(readString.toInt());
              break;
            default: 
              Serial.println("Invalid motor."); 
              Serial.println("\nEnter Command [Start, Stop, Release, StartAll, StopAll, ReleaseAll]");
              break; 
              //case 4:
              //mtrrun4=false; // stop motor 4
              //break;
          }
          readString = ""; //clears variable for new input
          progMode = 0;
          break;
        case 2:
          switch (readString.toInt()) {
            case 1:
              myStepper1->release();
              Serial.print("Releasing motor ");
              Serial.println(readString.toInt());
              break;
            case 2:
              myStepper2->release();
              Serial.print("Releasing motor ");
              Serial.println(readString.toInt());
              break;
            case 3:
              myStepper3->release();
              Serial.print("Releasing motor ");
              Serial.println(readString.toInt());
              break;
              //case 4:
              //myStepper4->release();
              //break;
            default: 
              Serial.println("Invalid motor"); 
          }
          readString = ""; //clears variable for new input
          progMode = 0;
          Serial.println("\nEnter Command [Start, Stop, Release, StartAll, StopAll, ReleaseAll]");
          break;
        case 3:
          if (readString.toInt() == 1 || readString.toInt() == 2 || readString.toInt() == 3) {
            Serial.print("Starting motor ");
            Serial.println(readString.toInt());
            mtr = readString.toInt();
            Serial.println("What speed?");
            progMode = 4;
          }
          else {
            progMode = 0; 
            Serial.println("Invalid motor"); 
            Serial.println("\nEnter Command [Start, Stop, Release, StartAll, StopAll, ReleaseAll]");
          }
          readString = ""; //clears variable for new input
          break;
        case 4:
          mtrspeed = readString.toInt();
          Serial.print("Starting motor ");
          Serial.print(mtr);
          Serial.print(" at speed ");
          Serial.println(mtrspeed);
          switch (mtr) {
            case 1:
              Astepper1.setSpeed(mtrspeed);
              mtrrun1 = true;
              break;
            case 2:
              Astepper2.setSpeed(mtrspeed);
              mtrrun2 = true;
              break;
            case 3:
              Astepper3.setSpeed(mtrspeed);
              mtrrun3 = true;
              break;
            //case 4:
              //Astepper4.setSpeed(mtrspeed);
              //mtrrun4 = true;
              //break;
          }
          readString = ""; //clears variable for new input
          progMode = 0;
          Serial.println("\nEnter Command [Start, Stop, Release, StartAll, StopAll, ReleaseAll]");
          break;
        case 5: 
          mtrspeed = readString.toInt(); 
          Serial.print("All motors to speed "); 
          Serial.println(mtrspeed); 
          Astepper1.setSpeed(mtrspeed); 
          Astepper2.setSpeed(mtrspeed); 
          Astepper3.setSpeed(mtrspeed); 
          //Astepper4.setSpeed(mtrspeed); 
          mtrrun1 = true; 
          mtrrun2 = true; 
          mtrrun3 = true; 
          //mtrrun4 = true; 
          readString = ""; 
          progMode = 0; 
          Serial.println("\nEnter Command [Start, Stop, Release, StartAll, StopAll, ReleaseAll]");
          break; 
      }
    }
  }
}

